% Function to factorize a matrix of input data (W) into the camera
% rotation matrix (R), translation (T), and the shape matrix (S).
% Three-dimensional version. Failure of normalization results in
% empty R and S. Align the R matrix with reference to the align frame 

function [R,T,S] = factorize(W, align)

pts = size(W,2);
T = W*ones(pts,1)/pts;

% Registered W. 
W = W - T*ones(1,pts);

% Use SVD to factor W. 
[u,lamda,v] = svd(W,0);

SVDvalues = diag(lamda)';
SVDvalues = SVDvalues(1:4)	% print first 4 SVD values

lamda1 = lamda(1:3,1:3);	% Since W is rank 3, b has only three meaningful values
root_lamda1 = sqrt(lamda1);

Rhat = u(:,1:3) * root_lamda1;
Shat = root_lamda1 * v(:,1:3)';

Q = Qmatrix(Rhat);

if size(Q,1) == 0,
   R = [];
   S = [];
else
  R = Rhat*Q;
  S = inv(Q)*Shat;

   %aligns the reference frame with the first camera 
  F = size(R,1)/2;

  R1 = R(align,:);
  R1 = R1/norm(R1);
  R2 = R(align+F,:);
  R2 = R2/norm(R2);
  R3 = cross(R1,R2);
  R3 = R3/norm(R3);
  P = [R1; R2; R3];
  P = P';

  R = R*P;
  S = inv(P)*S;
  
 
end
