#include "os.h"

#ifdef WINDOWS
#include <winsock.h>
#define snprintf _snprintf
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#define closesocket close
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void
httpget(char *host, int port, char *path, char buf[], int buflen, char **cont, int *contlen)
{
	static int init = 0;
	static struct sockaddr_in addr;

	#ifdef WINDOWS
	SOCKET sock;
	#else
	int sock;
	#endif

	int rv;

	int remlen;

	if (!init) {
		init = 1;

		addr.sin_family = AF_INET;
		addr.sin_port = htons(port);

		#ifdef WINDOWS
		if ((addr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
		#else
		if (inet_pton(AF_INET,host,&addr.sin_addr) == 0) {
		#endif
			fprintf(stderr,"httpget: invalid network address\n");
			exit(1);
		}
	}

	if ((sock = socket(PF_INET,SOCK_STREAM,0)) == -1) {
		perror("httpget: socket error");
		exit(1);
	}

	if (connect(sock,(struct sockaddr *)&addr,sizeof(addr)) == -1) {
		perror("httpget: connect error");
		closesocket(sock);
		exit(1);
	}

	rv = snprintf(buf,buflen,"GET %s HTTP/1.0\r\n\r\n",path);
	if (send(sock,buf,rv,0) != rv) {
		perror("httpget: send error");
		closesocket(sock);
		exit(1);
	}

	if ((rv = recv(sock,buf,buflen-1,0)) == -1) {
		perror("httpget: recv error");
		closesocket(sock);
		exit(1);
	}

	{
		char *p;

		if ((p = strstr(buf,"\r\nContent-Length: ")) == 0) {
			fprintf(stderr,"httpget: http response missing content-length\n");
			closesocket(sock);
			exit(1);
		}
		*contlen = atoi(p+18);

		if ((p = strstr(p+18,"\r\n\r\n")) == 0) {
			fprintf(stderr,"httpget: invalid http response\n");
			closesocket(sock);
			exit(1);
		}
		remlen = *contlen - (buf+rv - (p+4));
		*cont = p+4;
	}

	{
		char *p = buf+rv;

		while (remlen != 0) {
			if ((rv = recv(sock,p,remlen,0)) == -1) {
				perror("httpget: recv error");
				closesocket(sock);
				exit(1);
			}

			p += rv;
			remlen -= rv;

			if (p > buf+buflen) {
				fprintf(stderr,"httpget: buf is too short\n");
				closesocket(sock);
				exit(1);
			}
		}
	}

	*(*cont+*contlen) = 0;

	closesocket(sock);
}

/* FIXME perror in windows? */
/* FIXME perror/strerror is not thread-safe */

/* FIXME use keep-alive */
