%reconstruct: function to reconstruct a 3D shape
%depth: a 3*M matrix with M being the # of features points
%[xi,yi,zi] is the interpolated structure

function [xi,yi,zi]=reconstruct(feature_file, template_file, align)
color = pgmread(template_file);
[M,N] = size(color');
[xi,yi] = meshgrid(1:M, 1:N);
a = rmneg(load(feature_file));

[r,t,s] = factorize(a,align);
b = length(t)/2;
x = t(align);
y = t(align+b);
zi = griddata(s(1,:)+x, s(2,:)+y, s(3,:), xi, yi);
patch(surf2patch(xi, yi, zi,color)); shading flat; 
axis ij, colormap('gray'); grid on;

%plot the feature trajectories in the (X,Y,T) space

function plot_traj(in)

[M,T] = size(in);
M2 = M/2;
for t=1:T
   plot3(1:M2, in(1:M2,t), in(M2+1:M,t));
   hold on;
end
grid on;
xlabel('time instant');
ylabel('x-coordinate');
zlabel('y-coordinate');
title('feature trajectory');

%compute Q matrix using metric constraints

function G = Qmatrix(Rhat)

% number of frames
F = size(Rhat,1)/2;

clear Q
for f = 1:F,
  g = f + F;
  h = g + F;
  Q(f,:) = zt(Rhat(f,:), Rhat(f,:));
  Q(g,:) = zt(Rhat(g,:), Rhat(g,:));
  Q(h,:) = zt(Rhat(f,:), Rhat(g,:));
end

% Solve for v
rhs = [ones(2*F,1); zeros(F,1)];
v = Q \ rhs;

% C is a symmetric 3x3 matrix such that C = G * transpose(G)
C(1,1) = v(1);                  
C(1,2) = v(2);                  
C(1,3) = v(3);                  
C(2,2) = v(4);
C(2,3) = v(5);
C(3,3) = v(6);
C(2,1) = C(1,2);
C(3,1) = C(1,3);
C(3,2) = C(2,3);

e = eig(C);
neg = 0;
if e(1) <= 0, neg = 1; end
if e(2) <= 0, neg = 1; end
if e(3) <= 0, neg = 1; end
if neg == 1, G = [];
else G = sqrtm(C);
end

% zt(a,b)

function v = zt(a, b)

v = [	a(1)*b(1), a(1)*b(2)+a(2)*b(1), a(1)*b(3)+a(3)*b(1), ...
	a(2)*b(2), a(2)*b(3)+a(3)*b(2), a(3)*b(3) ];

