#include "httpget.h"
#include "jpegtoipl.h"
#include "os.h"
#include "webview.h"

#include <cv.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WINDOWS
#include <windows.h>
#define snprintf _snprintf
#else
#include <unistd.h>
#endif

IplImage *image;

int panspeedmin,panspeedmax, tiltspeedmin,tiltspeedmax,zoomspeedmin,zoomspeedmax;
int temptemp = 0;
int firstzz = 0;
static char connid[50];
static char path[200];

static char buf[20000];
static char *cont;
static int contlen;

int
timedout(char *buf, int main)
{
	char *p;

	if ((p = strstr(buf,"\r\nLivescope-Status: ")) == 0) {
		fprintf(stderr,"timedout: http response missing livescope-status\n");
		exit(1);
	}

	if (strncmp(p+20,"501 Unknown Connection ID",25) == 0) {
		if (main)
			OpenCameraServer("",-1);
		/*
		 * non-main thread expected to wait until timedout()==0
		 * FIXME probably better to use a semaphore against connid
		 */

		return 1;
	}
	return 0;
}

void
OpenCameraServer(char *host, int port)
{
	httpget(host,port,"/-wvhttp-01-/OpenCameraServer",buf,sizeof(buf),&cont,&contlen);

	if (strncmp(cont,"connection_id=",14) != 0) {
		fprintf(stderr,"OpenCameraServer: unexpected response\n");
		exit(1);
	}

	if (strlen(cont) > sizeof(connid)) {
		fprintf(stderr,"OpenCameraServer: connid is too short\n");
		exit(1);
	}

	strncpy(connid,cont,sizeof(connid));
	connid[strlen(connid)-1] = 0;
}

void
CloseCameraServer()
{
	snprintf(path,sizeof(path),"/-wvhttp-01-/CloseCameraServer?%s",connid);
	httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
}

void
GetLiveImage()
{
	snprintf(path,sizeof(path),"/-wvhttp-01-/GetLiveImage?%s",connid);
	httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

	if (timedout(buf,1)) {
		snprintf(path,sizeof(path),"/-wvhttp-01-/GetLiveImage?%s",connid);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
	}

	image = jpegtoipl(cont,contlen);
}

int
GetNotice(char buf[], int buflen, char **cont, int *contlen)
{
	char path[200];
	char *p, *q;

	snprintf(path,sizeof(path),"/-wvhttp-01-/GetNotice?%s",connid);
	httpget("",-1,path,buf,buflen,cont,contlen);

	while (timedout(buf,0)) {
		#ifdef WINDOWS
		Sleep(1000);
		#else
		usleep(1000000);
		#endif

		snprintf(path,sizeof(path),"/-wvhttp-01-/GetNotice?%s",connid);
		httpget("",-1,path,buf,buflen,cont,contlen);
	}

	if ((p = strstr(buf,"\r\nLivescope-Status: ")) == 0) {
		fprintf(stderr,"GetNotice: http response missing livescope-status\n");
		exit(1);
	}

	if (strncmp(p+20,"404 Operation Timeout",21) == 0)
		return 404;

	if ((q = strstr(buf,"\r\nLivescope-Notice: ")) == 0) {
		fprintf(stderr,"GetNotice: http response missing livescope-notice\n");
		exit(1);
	}

	return atoi(q+20);
}

void
GetCameraControl(char buf[], int buflen, char **cont, int *contlen)
{
	char path[200];

	snprintf(path,sizeof(path),"/-wvhttp-01-/GetCameraControl?%s",connid);
	httpget("",-1,path,buf,buflen,cont,contlen);

	while (timedout(buf,0)) {
		#ifdef WINDOWS
		Sleep(1000);
		#else
		usleep(1000000);
		#endif

		snprintf(path,sizeof(path),"/-wvhttp-01-/GetCameraControl?%s",connid);
		httpget("",-1,path,buf,buflen,cont,contlen);
	}
}

void
GetCameraControl_main()
{
	GetCameraControl(buf,sizeof(buf),&cont,&contlen);

	/*
	 * initialize camera speed variables
	 * safe to do here since this function is called from main thread
	 * and only expecting it to be called once at startup
	 */
	{
		char *str;
		char *p;

		snprintf(path,sizeof(path),"/-wvhttp-01-/GetPTZSpeedInfo?%s",connid);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

		#define parse(s,v) \
			str = "\n" #s "="; \
			if ((p = strstr(cont,str)) == 0) { \
				fprintf(stderr,"GetPTZSpeedInfo: http response missing " #s "\n"); \
				exit(1); \
			} \
			v = atoi(p+strlen(str));

		parse(direction_p_min,panspeedmin)
		parse(direction_p_max,panspeedmax)
		parse(direction_t_min,tiltspeedmin)
		parse(direction_t_max,tiltspeedmax)
        parse(direction_z_min,zoomspeedmin)
		parse(direction_z_max,zoomspeedmax)

	
		#undef parse
	}
}

void
ReleaseCameraControl()
{
	snprintf(path,sizeof(path),"/-wvhttp-01-/ReleaseCameraControl?%s",connid);
	httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
}

void
CenterCamera()
{
	int panmin,panmax, tiltmin,tiltmax, zoommin,zoommax;

	int p,t,z;

	/* get camera point motion limits */
	{
		char *str;
		char *p;

		snprintf(path,sizeof(path),"/-wvhttp-01-/GetCameraInfo?%s",connid);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

		#define parse(s,v) \
			str = "\n" #s "="; \
			if ((p = strstr(cont,str)) == 0) { \
				fprintf(stderr,"GetCameraInfo: http response missing " #s "\n"); \
				exit(1); \
			} \
			v = atoi(p+strlen(str));

		parse(pan_left_limit,panmin)
		parse(pan_right_limit,panmax)
		parse(tilt_up_limit,tiltmax)
		parse(tilt_down_limit,tiltmin)
		parse(zoom_tele_limit,zoommin)
		parse(zoom_wide_limit,zoommax)
	
		#undef parse
	}

	p = panmin + (panmax-panmin)/2;
	t = tiltmin + (tiltmax-tiltmin)/2;
	z = zoommin + (zoommax-zoommin)/2;

	snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCamera?%s&pan=%d&tilt=%d&zoom=%d",connid,p,t,z);
	httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
}

void
OperateCamera(double vx, double vy, double vz)
{
	
	char *dir = "stop";	
	char *str;
			char *p;
			int zz;
			int i= vz;

		int sx=abs((int)vx)*100, sy=abs((int)vy)*100, sz=abs((int)vz)*100;
        sz = sz<zoomspeedmin ? zoomspeedmin : (sz>zoomspeedmax ? zoomspeedmax : sz);
	
		sx = sx<panspeedmin ? panspeedmin : (sx>panspeedmax ? panspeedmax : sx);
		sy = sy<tiltspeedmin ? tiltspeedmin : (sy>tiltspeedmax ? tiltspeedmax : sy);
		//sz =

		snprintf(path,sizeof(path),"/-wvhttp-01-/SetPTZSpeed?%s&mode=direction&pan=%d&tilt=%d",connid,sx,sy);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
	
		if (timedout(buf,1)) {
			snprintf(path,sizeof(path),"/-wvhttp-01-/SetPTZSpeed?%s&mode=direction&pan=%d&tilt=%d",connid,sx,sy);
			httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
		}
	

	
	//	char *dir;

		if (vx==0 && vy==0)
			dir = "stop";

		else if (vx == 0)
			dir = vy>0 ? "up" : "down";

		else if (vy == 0)
			dir = vx>0 ? "right" : "left";

		else {
			if (vx>0 && vy>0)
				dir = "up_right";
			else if (vx>0 && vy<0)
				dir = "down_right";
			else if (vx<0 && vy>0)
				dir = "up_left";
			else //if (vx<0 && vx<0)
				dir = "down_left";
		}

		snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCameraEx?%s&pan_tilt=%s",connid,dir);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

		
	
   

			snprintf(path,sizeof(path),"/-wvhttp-01-/GetCameraInfo?%s",connid);
			httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

			#define parse(s,v) \
				str = "\n" #s "="; \
				if ((p = strstr(cont,str)) == 0) { \
					fprintf(stderr,"GetCameraInfo: http response missing " #s "\n"); \
					exit(1); \
				} \
				v = atoi(p+strlen(str));

			parse(zoom_current_value,zz)
		//	fprintf(stderr," zoom current value %d\n\n", zz);

			++temptemp;			
			if (temptemp > 1){
				//if ((vz-1)>0){
			if (fabs(vz - 1.0) > 0.3) {
				zz = firstzz * (vz );
				firstzz = zz;

				snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCamera?%s&zoom=%d",connid,zz);
				httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
			}
			else{ 
				firstzz = zz;
			//dir = "stop";
			}
			#undef parse
		}

	if (timedout(buf,1)) {
			snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCameraEx?%s&pan_tilt=%s",connid,dir);
			httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
	}


}

void
StopCamera()
{
	snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCameraEx?%s&pan_tilt=stop",connid);
	httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);

	if (timedout(buf,1)) {
		snprintf(path,sizeof(path),"/-wvhttp-01-/OperateCameraEx?%s&pan_tilt=stop",connid);
		httpget("",-1,path,buf,sizeof(buf),&cont,&contlen);
	}
}
