#include <cv.h>

/*
 * OpenCameraServer - establish a connection to a web camera at host
 *
 * CloseCameraServer - end a previously established connection
 *
 * GetLiveImage - retrieve a live image and store in the global image
 *
 * GetNotice - retrieve notifications of events
 *	for argument descriptions see httpget.h
 *	returns the event number as found in the Livescope-Notice field
 *	operates asynchronously (ie will block)
 *
 * GetCameraControl - request camera control
 *	for argument descriptions see httpget.h
 *	response returned by an event
 *
 * ReleaseCameraControl - release camera control
 *
 * OperateCamera - operate camera
 *  vx,y is the pan,tilt of the desired camera motion in degrees per second
 *  camera motion continues until the next OperateCamera or a boundary is reached
 *
 * CenterCamera - center camera within pan,tilt,zoom range
 *
 * the functions GetNotice and GetCameraControl are called from erthread
 * this is the reason why buf & friends need to be specified each call
 */

void OpenCameraServer(char *host, int port);
void CloseCameraServer();

extern IplImage *image;
void GetLiveImage();

int GetNotice(char buf[], int buflen, char **cont, int *contlen);

void GetCameraControl(char buf[], int buflen, char **cont, int *contlen);
void GetCameraControl_main(); /* only to be called from main thread */

void ReleaseCameraControl();

void OperateCamera(double vx, double vy);

void CenterCamera();
void StopCamera();

extern int panspeedmin,panspeedmax, tiltspeedmin,tiltspeedmax,zoomspeedmin,zoomspeedmax;
